@@ -373,6 +373,15 @@ merge = mergeBy (<)
 {- |
 'mergeBy' is like 'merge' but does not simply use the methods of the 'Ord' class
 but allows a custom comparison function.
+If in event lists @xs@ and @ys@ there are coinciding elements @x@ and @y@,
+and @cmp x y@ is 'True',
+then @x@ comes before @y@ in @mergeBy cmp xs ys@.
+
+> EventList> EventList.mergeBy (\_ _ -> True) (0 /. 'a' ./ empty) (0 /. 'b' ./ empty)
+> 0 /. 'a' ./ 0 /. 'b' ./ empty
+>
+> EventList> EventList.mergeBy (\_ _ -> False) (0 /. 'a' ./ empty) (0 /. 'b' ./ empty)
+> 0 /. 'b' ./ 0 /. 'a' ./ empty
 -}
 
 {-
