@@ -10,7 +10,7 @@ Event lists starting with a time difference and ending with a time difference.
 module Data.EventList.Relative.TimeTime
    (T,
     mapBody, mapTime,
-    zipWithBody, zipWithTime,
+    zipWithBody, zipWithTime, unzip,
     concatMapMonoid,
     traverse, traverse_, traverseBody, traverseTime,
     mapM, mapM_, mapBodyM, mapTimeM,
@@ -26,6 +26,7 @@ module Data.EventList.Relative.TimeTime
     mapMaybe, catMaybes, catMaybesR,
     append, concat, concatNaive, cycle, cycleNaive, reverse,
     splitAtTime, takeTime, dropTime,
+    forceTimeHead,
     discretize, resample,
     collectCoincident, flatten, mapCoincident,
     normalize, isNormalized,
@@ -47,9 +48,10 @@ import qualified Data.AlternatingList.List.Mixed as Mixed
 import qualified Data.List as List
 import qualified Data.EventList.Utility as Utility
 
-import Data.Monoid (Monoid, mconcat, mappend, )
+import Data.Monoid (Monoid, mempty, mconcat, )
 
 import qualified Numeric.NonNegative.Class as NonNeg
+import Numeric.NonNegative.Class ((-|), zero, add, )
 import Data.Tuple.HT (mapFst, mapSnd, mapPair, )
 import Data.Maybe.HT (toMaybe, )
 import Data.List.HT (isAscending, )
@@ -61,7 +63,7 @@ import Control.Applicative (Applicative, WrappedMonad(WrapMonad, unwrapMonad), )
 
 import Prelude hiding
    (null, foldr, foldl, map, filter, concat, cycle, reverse,
-    sequence, sequence_, mapM, mapM_)
+    sequence, sequence_, mapM, mapM_, unzip, )
 
 
 
@@ -79,8 +81,8 @@ getBodies = Uniform.getFirsts . decons
 getTimes :: T time body -> [time]
 getTimes = Uniform.getSeconds . decons
 
-duration :: Num time => T time body -> time
-duration = sum . getTimes
+duration :: NonNeg.C time => T time body -> time
+duration = NonNeg.sum . getTimes
 
 
 
@@ -132,6 +134,14 @@ zipWithTime ::
    (time0, [time0]) -> T time1 body -> T time2 body
 zipWithTime f = lift . Uniform.zipWithSecond f
 
+unzip :: T time (body0, body1) -> (T time body0, T time body1)
+unzip =
+   foldr
+      (\time ->
+         mapPair (consTime time, consTime time))
+      (\(body0, body1) ->
+         mapPair (consBody body0, consBody body1))
+      (mempty, mempty)
 
 
 concatMapMonoid :: Monoid m =>
@@ -210,51 +220,49 @@ Could be implemented using 'splitAt' and 'insert'.
 mergeBy :: (NonNeg.C time) =>
    (body -> body -> Bool) ->
    T time body -> T time body -> T time body
-mergeBy before xs0 ys0 =
-   let (xt,xs) = viewTimeL xs0
-       (yt,ys) = viewTimeL ys0
-   in  case compare xt yt of
-          LT -> mergeFirstBy before xs0 ys0
-          GT -> mergeFirstBy before ys0 xs0
-          EQ ->
-             consTime xt $
-             case (viewBodyL xs, viewBodyL ys) of
-                (Nothing, _) -> ys
-                (_, Nothing) -> xs
-                (Just (b0,xs1), Just (b1,ys1)) ->
-                   {- do not insert both b0 and b1 immediately,
-                      because the later one of b0 and b1 may be pushed even further,
-                      thus recourse with 'mergeBy' on xs or ys -}
-                   if before b0 b1
-                     then consBody b0 $ mergeBy before xs1 $ consTime 0 ys
-                     else consBody b1 $ mergeBy before ys1 $ consTime 0 xs
-
-{- | merge two time ordered lists provided that e0 is earlier than e1 -}
-mergeFirstBy :: (NonNeg.C time) =>
-   (body -> body -> Bool) ->
-   T time body -> T time body -> T time body
-mergeFirstBy before xs0 ys0 =
-   let (xt,xs) = viewTimeL xs0
-       (yt,ys) = viewTimeL ys0
-   in  switchBodyL
-          ys0
-          (\ b xs1 ->
-              consTime xt $ consBody b $ mergeBy before xs1 $ consTime (yt-xt) ys)
-          xs
+mergeBy before =
+   let recourse xs0 ys0 =
+          let (xt,xs) = viewTimeL xs0
+              (yt,ys) = viewTimeL ys0
+              (mt,~(bef,dt)) = NonNeg.split xt yt
+          in  delay mt $
+              if dt == zero
+                then
+                   case (viewBodyL xs, viewBodyL ys) of
+                      (Nothing, _) -> consTime zero ys
+                      (_, Nothing) -> consTime zero xs
+                      (Just (b0,xs1), Just (b1,ys1)) ->
+                         {-
+                         do not insert both b0 and b1 immediately,
+                         because the later one of b0 and b1 may be pushed even further,
+                         thus recourse with 'mergeBy' on xs or ys
+                         -}
+                         if before b0 b1
+                           then cons zero b0 $
+                                recourse xs1 (consTime zero ys)
+                           else cons zero b1 $
+                                recourse (consTime zero xs) ys1
+                else
+                  if bef
+                    then
+                       let ys1 = consTime dt ys
+                       in  flip (switchBodyL ys1) xs $ \ b xs1 ->
+                              cons zero b $ recourse xs1 ys1
+                    else
+                       let xs1 = consTime dt xs
+                       in  flip (switchBodyL xs1) ys $ \ b ys1 ->
+                              cons zero b $ recourse xs1 ys1
+   in  recourse
 
 
 {- |
 Note that 'merge' compares entire events rather than just start
 times.  This is to ensure that it is commutative, a desirable
-condition for some of the proofs used in \secref{equivalence}.
+condition for some of the proofs used in Haskore/section equivalence.
 It is also necessary to assert a unique representation
-of the performance independent of the structure of the 'Music.T note'.
+of the event list independent of the structure of the event type.
 The same function for inserting into a time ordered list with a trailing pause.
-The strictness annotation is necessary for working with infinite lists.
-
-Here are two other functions that are already known for non-padded time lists.
 -}
-
 insert :: (NonNeg.C time, Ord body) =>
    time -> body -> T time body -> T time body
 insert = insertBy (<)
@@ -268,18 +276,19 @@ insertBy :: (NonNeg.C time) =>
    time -> body -> T time body -> T time body
 insertBy before t0 me0 =
    let recurseTime t =
-          switchTimeL (\ t1 xs ->
-             if t<t1
-               then cons t me0 (consTime (t1-t) xs)
-               else recurseBody t1 t xs)
-       recurseBody t1 t =
-          switchBodyL
-             (cons t me0 $ pause 0)
-             (\ me1 xs ->
-                 consTime t1 $
-                    if t==t1 && before me0 me1
-                      then consBody me0 (cons 0 me1 xs)
-                      else consBody me1 (recurseTime (t-t1) xs))
+          switchTimeL $ \ t1 xs0 ->
+             let (mt,~(b,dt)) = NonNeg.split t1 t
+             in  delay mt $
+                 if not b
+                   then cons zero me0 $ consTime dt xs0
+                   else
+                     switchBodyL
+                        (cons dt me0 $ pause zero)
+                        (\ me1 xs -> consTime zero $
+                           if dt==zero && before me0 me1
+                             then consBody me0 (cons zero me1 xs)
+                             else consBody me1 (recurseTime dt xs))
+                        xs0
    in   recurseTime t0
 
 
@@ -297,7 +306,7 @@ Move events towards the front of the event list.
 You must make sure, that no event is moved before time zero.
 This works only for finite lists.
 -}
-moveForward :: (NonNeg.C time) =>
+moveForward :: (Ord time, Num time) =>
    T time (time, body) -> T time body
 moveForward =
    fromAbsoluteEventList .
@@ -311,7 +320,7 @@ moveBackward =
    foldr
       (\t -> cons t Nothing)
       (\(t,b) -> insertBy (ltMaybe (\_ _ -> True)) t (Just b))
-      (pause 0)
+      (pause zero)
 
 {- |
 Like 'moveForward' but restricts the look-ahead time.
@@ -332,7 +341,7 @@ moveForwardRestricted :: (Ord body, NonNeg.C time) =>
 moveForwardRestricted maxTime =
    decreaseStart maxTime .
    moveBackward .
-   mapBody (mapFst (maxTime-)) .
+   mapBody (mapFst (maxTime-|)) .
    pad maxTime
 {-
    moveForwardRestrictedBy
@@ -357,7 +366,7 @@ moveForwardRestrictedBy cmp maxTime =
    catMaybes .
    foldr
       (\t -> cons t Nothing)
-      (\(t,b) -> insertBy (ltMaybe cmp) (maxTime-t) (Just b))
+      (\(t,b) -> insertBy (ltMaybe cmp) (maxTime-|t) (Just b))
       (pause maxTime)
 
 -- | currently only for testing
@@ -368,11 +377,11 @@ moveForwardRestrictedByStrict cmp maxTime =
    decreaseStart maxTime .
    foldr
       delay
-      (\(t,b) -> insertBy cmp (maxTime-t) b)
+      (\(t,b) -> insertBy cmp (maxTime-|t) b)
       (pause maxTime)
 
 -- | currently only for testing
-moveForwardRestrictedByQueue :: (NonNeg.C time) =>
+moveForwardRestrictedByQueue :: (NonNeg.C time, Num time) =>
    (body -> body -> Bool) ->
    time -> T time (time, body) -> T time body
 moveForwardRestrictedByQueue cmp maxTime xs =
@@ -399,7 +408,7 @@ Merge several event lists respecting the start time of the outer event list.
 -}
 arrange :: (Ord body, NonNeg.C time) =>
    T time (T time body) -> T time body
-arrange = arrangeBy (\_ _ -> False)
+arrange = arrangeBy (\_ _ -> True)
 
 arrangeBy :: (NonNeg.C time) =>
    (body -> body -> Bool) ->
@@ -409,16 +418,9 @@ arrangeBy cmp =
    foldr
       (\t -> cons t Nothing)
       (\xs -> mergeBy (ltMaybe cmp) (mapBody Just xs))
-      (pause 0)
+      (pause zero)
 
 
-{-
-not lazy enough for @append (2 /. 'a' ./ 4 /. 'b' ./ 2 /. undefined) undefined@
--}
-append :: (NonNeg.C time) =>
-   T time body -> T time body -> T time body
-append = mappend
-
 concat :: (NonNeg.C time) =>
    [T time body] -> T time body
 concat = mconcat
@@ -433,7 +435,7 @@ Thus we detect subsequent empty lists and merge them from left to right.
 -}
 concatNaive :: (NonNeg.C time) =>
    [T time body] -> T time body
-concatNaive = List.foldr append (pause 0)
+concatNaive = List.foldr append (pause zero)
 
 
 {- |
@@ -446,7 +448,7 @@ cycle =
    (\t0 xs ->
        consTime t0 $
        BodyTimePriv.cycle $
-       BodyTimePriv.mapTimeLast (+t0) xs)
+       BodyTimePriv.mapTimeLast (add t0) xs)
 
 
 cycleNaive :: (NonNeg.C time) =>
@@ -472,13 +474,19 @@ splitAtTime :: (NonNeg.C time) =>
 splitAtTime t0 =
    switchTimeL
    (\t1 xs ->
-        if t0<=t1
-          then (pause t0, consTime (t1-t0) xs)
-          else
-             switchBodyL
-                (pause t1, pause 0)
-                (\ b -> mapFst (cons t1 b) . splitAtTime (t0-t1))
-                xs)
+      let (mt,~(bef,dt)) = NonNeg.split t0 t1
+      in  {-
+          The handling of the second pair member looks a bit cumbersome,
+          but it is necessary to prepend the time once
+          in order to prevent a memory leak.
+          -}
+          mapPair (consTime mt, forceTimeHead) $
+          if bef
+            then (mempty, consTime dt xs)
+            else switchBodyL
+                    (mempty, pause zero)
+                    (\ b -> mapFst (consBody b) . splitAtTime dt)
+                    xs)
 
 takeTime :: (NonNeg.C time) =>
    time -> T time body -> T time body
@@ -486,13 +494,42 @@ takeTime t = fst . splitAtTime t
 
 dropTime :: (NonNeg.C time) =>
    time -> T time body -> T time body
-dropTime t = snd . splitAtTime t
+-- dropTime t = snd . splitAtTime t
+dropTime t0 =
+   switchTimeL
+   (\t1 xs ->
+      let (bef,dt) = snd $ NonNeg.split t0 t1
+      in  forceTimeHead $
+          if bef
+            then consTime dt xs
+            else switchBodyL
+                    (pause zero)
+                    (\ _b -> dropTime dt)
+                    xs)
+
+{-
+Surprisingly this has a space leak,
+see test dropTimeLazyInfinite.
+
+dropTime :: (NonNeg.C time) =>
+   time -> T time body -> T time body
+dropTime t0 =
+   switchTimeL
+   (\t1 xs ->
+      let (bef,dt) = snd $ NonNeg.split t0 t1
+      in  if bef
+            then consTime dt xs
+            else switchBodyL
+                    (pause zero)
+                    (\ _b -> dropTime dt)
+                    xs)
+-}
 
 
 decreaseStart :: (NonNeg.C time) =>
    time -> T time body -> T time body
 decreaseStart dif =
-   mapTimeHead (subtract dif)
+   mapTimeHead (-| dif)
 
 
 collectCoincident :: (NonNeg.C time) => T time body -> T time [body]
@@ -514,12 +551,12 @@ where the clever implementation sums up pauses
 from the beginning to the end.
 -}
 
-filter :: (Num time) =>
+filter :: (NonNeg.C time) =>
    (body -> Bool) ->
    T time body -> T time body
 filter p = mapMaybe (\b -> toMaybe (p b) b)
 
-mapMaybe :: (Num time) =>
+mapMaybe :: (NonNeg.C time) =>
    (body0 -> Maybe body1) ->
    T time body0 -> T time body1
 mapMaybe f = catMaybes . mapBody f
@@ -528,48 +565,50 @@ mapMaybe f = catMaybes . mapBody f
 Adds times in a left-associative fashion.
 Use this if the time is a strict data type.
 -}
-catMaybes :: (Num time) =>
+catMaybes :: (NonNeg.C time) =>
    T time (Maybe body) -> T time body
-catMaybes = mapTime sum . lift Uniform.catMaybesFirst
+catMaybes = mapTime NonNeg.sum . lift Uniform.catMaybesFirst
 
 {- |
 Adds times in a right-associative fashion.
 Use this if the time is a data type like lazy Peano numbers
 or "Numeric.NonNegative.Chunky".
 -}
-catMaybesR :: (Num time) =>
+catMaybesR :: (NonNeg.C time) =>
    T time (Maybe body) -> T time body
 catMaybesR =
    foldr
-      (mapTimeHead . (+))
-      (maybe id (cons 0))
-      (pause 0)
+      (mapTimeHead . add)
+      (maybe id (cons zero))
+      (pause zero)
 
-partition :: (Num time) =>
+partition :: (NonNeg.C time) =>
    (body -> Bool) ->
    T time body -> (T time body, T time body)
 partition p =
-   mapPair (mapTime sum, mapTime sum) .
+   mapPair (mapTime NonNeg.sum, mapTime NonNeg.sum) .
    mapPair (Cons, Cons) .
    Uniform.partitionFirst p .
    decons
 
-partitionMaybe :: (Num time) =>
+partitionMaybe :: (NonNeg.C time) =>
    (body0 -> Maybe body1) -> T time body0 ->
    (T time body1, T time body0)
 partitionMaybe f =
-   mapPair (mapTime sum . Cons, mapTime sum . Cons) .
+   mapPair (mapTime NonNeg.sum . Cons, mapTime NonNeg.sum . Cons) .
    Uniform.partitionMaybeFirst f .
    decons
 
 {- |
 Cf. 'catMaybesR'
 -}
-partitionMaybeR :: (Num time) =>
+partitionMaybeR :: (NonNeg.C time) =>
    (body0 -> Maybe body1) -> T time body0 ->
    (T time body1, T time body0)
 partitionMaybeR f =
-   mapPair (mapTime (List.foldr (+) 0), mapTime (List.foldr (+) 0)) .
+   mapPair
+      (mapTime (List.foldr add zero),
+       mapTime (List.foldr add zero)) .
    mapPair (Cons, Cons) .
    Uniform.partitionMaybeFirst f .
    decons
@@ -578,14 +617,11 @@ partitionMaybeR f =
 Since we need it later for MIDI generation,
 we will also define a slicing into equivalence classes of events.
 -}
-slice :: (Eq a, Num time) =>
+slice :: (Eq a, NonNeg.C time) =>
    (body -> a) -> T time body -> [(a, T time body)]
 slice = Utility.slice (fmap fst . viewBodyL . snd . viewTimeL) partition
 
 
-foldr :: (time -> a -> b) -> (body -> b -> a) -> a -> T time body -> b
-foldr f g x = Uniform.foldr g f x . decons
-
 foldl :: (a -> time -> b) -> (b -> body -> a) -> a -> T time body -> b
 foldl f g x = Uniform.foldl g f x . decons
 
