@@ -24,7 +24,6 @@ import Control.Monad (liftM)
 
 import qualified Numeric.NonNegative.Class as NonNeg
 import Data.Tuple.HT (mapFst, mapSnd, mapPair, )
-import qualified Control.Monad as Monad
 import Control.Monad.Trans.State (state, evalState)
 
 import Prelude hiding (filter, concat)
@@ -38,7 +37,7 @@ infixl 5 $~
 
 infixl 4 ==~
 
-(==~) :: (Eq body, Num time) =>
+(==~) :: (Eq body, Eq time, Num time) =>
    AbsTime.T time body -> RelTime.T time body -> Bool
 (==~) xs ys =
    xs == RelTime.toAbsoluteEventList 0 ys
