@@ -26,7 +26,7 @@ import Data.EventList.Relative.TimeTimePrivate (($~~), lift, )
 
 import qualified Numeric.NonNegative.Chunky as NonNegChunky
 import qualified Numeric.NonNegative.Class  as NonNeg
-import Numeric.NonNegative.Class ((-|), )
+import Numeric.NonNegative.Class ((-|), zero, add, )
 import Data.EventList.Relative.TimeTime (isNormalized, )
 
 import System.Random (Random, randomR, mkStdGen, )
@@ -152,9 +152,86 @@ consInfix :: (NonNeg.C time, Eq body) =>
    time -> body -> time -> time -> body -> time -> Bool
 consInfix t0a b0 t0b t1a b1 t1b =
    TimeTimeList.append (t0a /. b0 ./ t0b /. empty) (t1a /. b1 ./ t1b /. empty)
-      == (t0a /. b0 ./ (t0b+t1a) /. b1 ./ t1b /. empty)
+      == (t0a /. b0 ./ (add t0b t1a) /. b1 ./ t1b /. empty)
 
 
+iterate' :: (a -> a) -> a -> [a]
+iterate' f =
+   let recourse x = ((:) $! x) $ recourse (f x)
+   in  recourse
+
+chunkyShow ::
+   Int -> Bool
+chunkyShow =
+   (\t -> t==t) . take 1000000 . show .
+   const (NonNegChunky.fromChunks $ iterate' (2-) (1::TimeDiff))
+
+_chunkyCheck ::
+   Int -> Bool
+_chunkyCheck =
+   (\t -> t==t) .
+   take 1000000 .
+--   (!!1000000) .
+{-
+   NonNegChunky.toChunks .
+   NonNegChunky.fromChunks .
+-}
+   iterate' (1+)
+
+{-
+With an early implementation of mapTimeTail this resulted in heap exhaustion.
+-}
+mapTimeTailChunkyInfinite ::
+   (NonNeg.C time, Num time, Eq body, Show body) =>
+   (body -> body) ->
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+mapTimeTailChunkyInfinite _f =
+   (\t -> t==t) . take 1000000 .
+   NonNegChunky.toChunks .
+   MixedTimeList.switchTimeL const .
+{-
+   MixedTimeList.switchTimeL
+      MixedTimeList.consTime .
+-}
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate' (2-) 1)
+
+{-
+mapTimeTailChunkyInfinite ::
+   (NonNeg.C time, Num time, Eq body, Show body) =>
+   (body -> body) ->
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+mapTimeTailChunkyInfinite f =
+   (\t -> t==t) . take 1000000 . show .
+   MixedTimeList.mapTimeTail (fmap f) .
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+-}
+
+{-
+mapTimeTailChunkyInfinite :: (NonNeg.C time, Eq body) =>
+   (body -> body) ->
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+mapTimeTailChunkyInfinite f =
+--   not . NonNegChunky.isNull .
+--   not . null . NonNegChunky.toChunks .
+   (\t -> t==t) . take 1000000 . NonNegChunky.toChunks .
+   MixedTimeList.switchTimeL const .
+--   TimeTimeList.dropTime 1000000 .
+   MixedTimeList.mapTimeTail (fmap f) .
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+
+
+mapTimeTailChunkyInfinite :: (NonNeg.C time, Eq body) =>
+   (body -> body) ->
+   NonNegChunky.T time ->
+   TimeTimeList.T (NonNegChunky.T time) body -> Bool
+mapTimeTailChunkyInfinite f time =
+   MixedTimeList.mapTimeTail (fmap f) .
+   TimeTimeList.delay
+      (let infTime = mappend time infTime in infTime)
+-}
 
 
 mapBodyComposition :: (Eq body2, Eq time) =>
@@ -203,12 +280,12 @@ mapNormalize f =
 appendLeftIdentity :: (NonNeg.C time, Eq body) =>
    TimeTimeList.T time body -> Bool
 appendLeftIdentity xs =
-   TimeTimeList.append (TimeTimeList.pause 0) xs  ==  xs
+   TimeTimeList.append (TimeTimeList.pause zero) xs  ==  xs
 
 appendRightIdentity :: (NonNeg.C time, Eq body) =>
    TimeTimeList.T time body -> Bool
 appendRightIdentity xs =
-   TimeTimeList.append xs (TimeTimeList.pause 0)  ==  xs
+   TimeTimeList.append xs (TimeTimeList.pause zero)  ==  xs
 
 appendAssociative :: (NonNeg.C time, Eq body) =>
    TimeTimeList.T time body -> TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
@@ -220,7 +297,7 @@ appendCons :: (NonNeg.C time, Eq body) =>
    time -> body -> TimeTimeList.T time body -> Bool
 appendCons time body xs =
    TimeTimeList.cons time body xs  ==
-   TimeTimeList.append (TimeTimeList.cons time body (TimeTimeList.pause 0)) xs
+   TimeTimeList.append (TimeTimeList.cons time body (TimeTimeList.pause zero)) xs
 
 appendSplitAtTime :: (NonNeg.C time, Eq body) =>
    time -> TimeTimeList.T time body -> Bool
@@ -261,13 +338,13 @@ decreaseStartInfinite dif =
 delayAdditive :: (NonNeg.C time, Eq body) =>
    time -> time -> TimeTimeList.T time body -> Bool
 delayAdditive dif0 dif1 xs =
-   TimeTimeList.delay (dif0+dif1) xs ==
+   TimeTimeList.delay (add dif0 dif1) xs ==
    TimeTimeList.delay dif0 (TimeTimeList.delay dif1 xs)
 
 delayPause :: (NonNeg.C time) =>
    time -> time -> Bool
 delayPause dif0 dif1 =
-   let pause = TimeTimeList.pause (dif0+dif1)
+   let pause = TimeTimeList.pause (add dif0 dif1)
    in  TimeTimeList.delay dif0 (TimeTimeList.pause dif1) ==
        (asTypeOf pause (TimeTimeList.cons dif0 () pause))
 
@@ -294,9 +371,9 @@ splitAtTakeDropTime t xs =
 takeTimeEndPause :: (NonNeg.C time, Ord body) =>
    time -> TimeTimeList.T time body -> Bool
 takeTimeEndPause t xs =
-   t == 0 ||
+   t == zero ||
    t >= TimeTimeList.duration xs ||
-   0 <  snd (TimeMixedList.viewTimeR (TimeTimeList.takeTime t xs))
+   zero <  snd (TimeMixedList.viewTimeR (TimeTimeList.takeTime t xs))
 
 takeTimeAppendFirst :: (NonNeg.C time, Eq body) =>
    time -> TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
@@ -306,7 +383,7 @@ takeTimeAppendFirst t xs ys =
       (TimeTimeList.takeTime t xs)
       (TimeTimeList.takeTime (t -| TimeTimeList.duration xs) ys)
 
-takeTimeAppendSecond :: (NonNeg.C time, Eq body) =>
+takeTimeAppendSecond :: (NonNeg.C time, Num time, Eq body) =>
    time -> TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
 takeTimeAppendSecond t xs0 ys =
    -- the first list must not end with a zero pause
@@ -324,17 +401,55 @@ dropTimeNormalize :: (NonNeg.C time, Ord body) =>
 dropTimeNormalize t =
    isNormalized . TimeTimeList.dropTime t . TimeTimeList.normalize
 
-takeTimeInfinite :: (NonNeg.C time, Ord body) =>
+takeTimeInfinite :: (NonNeg.C time, Num time, Ord body) =>
    time -> NonEmptyList time body -> Bool
 takeTimeInfinite t =
    (t == ) . TimeTimeList.duration .
    TimeTimeList.takeTime t . makeUncollapsedInfiniteEventList
 
-dropTimeInfinite :: (NonNeg.C time, Ord body) =>
+dropTimeInfinite :: (NonNeg.C time, Num time, Ord body) =>
    time -> NonEmptyList time body -> Bool
 dropTimeInfinite t =
    checkInfinite . TimeTimeList.dropTime t . makeUncollapsedInfiniteEventList
 
+dropTimeLargeInfinite ::
+   (NonNeg.C time, Num time, Ord body) =>
+   NonEmptyList time body -> Bool
+dropTimeLargeInfinite =
+   checkInfinite .
+   TimeTimeList.dropTime 10000 .
+   makeUncollapsedInfiniteEventList
+
+
+splitAtTimeLazyInfinite ::
+   (NonNeg.C time, Num time, Ord body, Show time, Show body) =>
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+splitAtTimeLazyInfinite =
+   not . null . show . snd .
+   TimeTimeList.splitAtTime 1000000 .
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+
+dropTimeLazyInfinite ::
+   (NonNeg.C time, Num time, Ord body, Show time, Show body) =>
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+dropTimeLazyInfinite =
+   not . null . show .
+   TimeTimeList.dropTime 1000000 .
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+
+{-
+dropTimeLazyInfinite ::
+   (NonNeg.C time, Num time, Ord body) =>
+   BodyTimeList.T (NonNegChunky.T time) body -> Bool
+dropTimeLazyInfinite =
+   (\t -> t==t) . take 100 . NonNegChunky.toChunks .
+   MixedTimeList.switchTimeL const .
+   TimeTimeList.dropTime 1000000 .
+   MixedTimeList.consTime
+      (NonNegChunky.fromChunks $ iterate (2-) 1)
+-}
 
 
 
@@ -347,7 +462,7 @@ durationAppend :: (NonNeg.C time) =>
    TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
 durationAppend xs ys =
    TimeTimeList.duration (TimeTimeList.append xs ys)  ==
-   TimeTimeList.duration xs + TimeTimeList.duration ys
+   TimeTimeList.duration xs `add` TimeTimeList.duration ys
 
 durationMerge :: (NonNeg.C time, Ord body) =>
    TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
@@ -394,20 +509,20 @@ cycleInfinite xs0 =
    in  equalPrefix 100 xs (TimeTimeList.cycle xs)
 
 
-filterSatisfy :: (Num time) =>
+filterSatisfy :: (NonNeg.C time) =>
    (body -> Bool) ->
    TimeTimeList.T time body -> Bool
 filterSatisfy p =
    all p . TimeTimeList.getBodies . TimeTimeList.filter p
 
-filterProjection :: (Num time, Eq body) =>
+filterProjection :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    TimeTimeList.T time body -> Bool
 filterProjection p xs =
    TimeTimeList.filter p xs ==
    TimeTimeList.filter p (TimeTimeList.filter p xs)
 
-filterCommutative :: (Num time, Eq body) =>
+filterCommutative :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    (body -> Bool) ->
    TimeTimeList.T time body -> Bool
@@ -415,7 +530,7 @@ filterCommutative p q xs =
    TimeTimeList.filter p (TimeTimeList.filter q xs) ==
    TimeTimeList.filter q (TimeTimeList.filter p xs)
 
-filterComposition :: (Num time, Eq body) =>
+filterComposition :: (NonNeg.C time, Eq body) =>
    (body -> Bool) ->
    (body -> Bool) ->
    TimeTimeList.T time body -> Bool
@@ -464,23 +579,23 @@ catMaybesAppend xs ys =
    TimeTimeList.append (TimeTimeList.catMaybes xs) (TimeTimeList.catMaybes ys)
 
 
-catMaybesRInfinite :: (NonNeg.C time, Eq body) =>
+catMaybesRInfinite :: (NonNeg.C time, Num time, Eq body) =>
    NonEmptyList time (Maybe body) -> Bool
 catMaybesRInfinite xs =
    {-
-   @(1+) is needed in order to assert that the accumulated time is infinite
+   @(add 1) is needed in order to assert that the accumulated time is infinite
    and can be clipped by @min 100@.
    -}
    let t =
          min 100 $
-         List.foldr (+) 0 $
+         List.foldr add zero $
          TimeTimeList.getTimes $
          TimeTimeList.catMaybesR $
-         TimeTimeList.mapTime (NonNegChunky.fromNumber . (1+)) $
+         TimeTimeList.mapTime (NonNegChunky.fromNumber . (add 1)) $
          makeInfiniteEventList xs
    in  t == t
 
-catMaybesRInitInfinite :: (NonNeg.C time, Eq body) =>
+catMaybesRInitInfinite :: (NonNeg.C time, Num time, Eq body) =>
    NonEmptyList time body -> Bool
 catMaybesRInitInfinite xs =
    let t =
@@ -488,7 +603,7 @@ catMaybesRInitInfinite xs =
          MixedTimeList.switchTimeL const $
          TimeTimeList.catMaybesR $
          TimeTimeList.mapBody (const (Nothing::Maybe())) $
-         TimeTimeList.mapTime (NonNegChunky.fromNumber . (1+)) $
+         TimeTimeList.mapTime (NonNegChunky.fromNumber . (add 1)) $
          makeInfiniteEventList xs
    in  t == t
 
@@ -506,27 +621,27 @@ partitionMaybeR f xs =
    TimeTimeList.partitionMaybeR f xs
 
 partitionMaybeRInfinite ::
-   (NonNeg.C time, Eq body0, Eq body1) =>
+   (NonNeg.C time, Num time, Eq body0, Eq body1) =>
    (body0 -> Maybe body1) ->
    NonEmptyList time body0 -> Bool
 partitionMaybeRInfinite f xs =
    {-
-   @(1+) is needed in order to assert that the accumulated time is infinite
+   @(add 1) is needed in order to assert that the accumulated time is infinite
    and can be clipped by @min 100@.
    -}
    let timeSum =
          min 100 .
-         List.foldr (+) 0 .
+         List.foldr add zero .
          TimeTimeList.getTimes
        t =
          mapPair (timeSum, timeSum) $
          TimeTimeList.partitionMaybeR f $
-         TimeTimeList.mapTime (NonNegChunky.fromNumber . (1+)) $
+         TimeTimeList.mapTime (NonNegChunky.fromNumber . add 1) $
          makeInfiniteEventList xs
    in  t == t
 
 partitionMaybeRInitInfinite ::
-   (NonNeg.C time, Eq body0, Eq body1) =>
+   (NonNeg.C time, Num time, Eq body0, Eq body1) =>
    (body0 -> Maybe body1) ->
    NonEmptyList time body0 -> Bool
 partitionMaybeRInitInfinite f xs =
@@ -536,7 +651,7 @@ partitionMaybeRInitInfinite f xs =
        t =
          mapPair (initTime, initTime) $
          TimeTimeList.partitionMaybeR f $
-         TimeTimeList.mapTime (NonNegChunky.fromNumber . (1+)) $
+         TimeTimeList.mapTime (NonNegChunky.fromNumber . add 1) $
          makeInfiniteEventList xs
    in  t == t
 
@@ -554,12 +669,12 @@ mergeNormalize xs0 ys0 =
 mergeLeftIdentity :: (NonNeg.C time, Ord body) =>
    TimeTimeList.T time body -> Bool
 mergeLeftIdentity xs =
-   TimeTimeList.merge (TimeTimeList.pause 0) xs  ==  xs
+   TimeTimeList.merge (TimeTimeList.pause zero) xs  ==  xs
 
 mergeRightIdentity :: (NonNeg.C time, Ord body) =>
    TimeTimeList.T time body -> Bool
 mergeRightIdentity xs =
-   TimeTimeList.merge xs (TimeTimeList.pause 0)  ==  xs
+   TimeTimeList.merge xs (TimeTimeList.pause zero)  ==  xs
 
 mergeCommutative :: (NonNeg.C time, Ord body) =>
    TimeTimeList.T time body -> TimeTimeList.T time body -> Bool
@@ -698,7 +813,7 @@ insertMerge :: (NonNeg.C time, Ord body) =>
 insertMerge time body evs0 =
    let evs = TimeTimeList.normalize evs0
    in  TimeTimeList.insert time body evs  ==
-       TimeTimeList.merge (TimeTimeList.cons time body $ TimeTimeList.pause 0) evs
+       TimeTimeList.merge (TimeTimeList.cons time body $ TimeTimeList.pause zero) evs
 
 insertNormalize :: (NonNeg.C time, Ord body) =>
    time -> body -> TimeTimeList.T time body -> Bool
@@ -714,7 +829,7 @@ insertSplitAtTime time body evs =
    ==
       let (prefix,suffix) = TimeTimeList.splitAtTime time evs
       in  TimeTimeList.normalize
-             (TimeTimeList.append prefix (TimeTimeList.cons 0 body suffix))
+             (TimeTimeList.append prefix (TimeTimeList.cons zero body suffix))
       --  append prefix (MixedTimeList.consBody body suffix)
 
 insertInfinite :: (NonNeg.C time, Ord body) =>
@@ -725,21 +840,21 @@ insertInfinite time body =
 
 
 
-moveForwardIdentity :: (NonNeg.C time, Ord body) =>
+moveForwardIdentity :: (NonNeg.C time, Num time, Ord body) =>
    TimeTimeList.T time body -> Bool
 moveForwardIdentity evs =
    evs ==
-   TimeTimeList.moveForward (TimeTimeList.mapBody ((,) 0) evs)
+   TimeTimeList.moveForward (TimeTimeList.mapBody ((,) zero) evs)
 
-moveForwardAdditive :: (NonNeg.C time, Ord body) =>
+moveForwardAdditive :: (NonNeg.C time, Num time, Ord body) =>
    TimeTimeList.T time ((time,time),body) -> Bool
 moveForwardAdditive evs =
    TimeTimeList.normalize (moveForwardLimited (moveForwardLimited
       (TimeTimeList.mapBody (\((t0,t1),b) -> (t0,(t1,b))) evs))) ==
    TimeTimeList.normalize (moveForwardLimited
-      (TimeTimeList.mapBody (mapFst (uncurry (+))) evs))
+      (TimeTimeList.mapBody (mapFst (uncurry add)) evs))
 
-moveForwardCommutative :: (NonNeg.C time, Ord body) =>
+moveForwardCommutative :: (NonNeg.C time, Num time, Ord body) =>
    TimeTimeList.T time ((time,time),body) -> Bool
 moveForwardCommutative evs =
    TimeTimeList.normalize (moveForwardLimited (moveForwardLimited
@@ -747,7 +862,7 @@ moveForwardCommutative evs =
    TimeTimeList.normalize (moveForwardLimited (moveForwardLimited
       (TimeTimeList.mapBody (\((t0,t1),b) -> (t1,(t0,b))) evs)))
 
-moveForwardRestricted :: (NonNeg.C time, Ord body) =>
+moveForwardRestricted :: (NonNeg.C time, Num time, Ord body) =>
    time -> TimeTimeList.T time (time,body) -> Bool
 moveForwardRestricted maxTime evs0 =
    let evs =
@@ -758,7 +873,8 @@ moveForwardRestricted maxTime evs0 =
        &&  mevs  ==  TimeTimeList.moveForwardRestrictedByStrict (\_ _ -> True) maxTime evs
        &&  mevs  ==  TimeTimeList.moveForwardRestrictedByQueue (\_ _ -> False) maxTime evs
 
-moveForwardRestrictedInfinity :: (NonNeg.C time, Ord body) =>
+moveForwardRestrictedInfinity ::
+   (NonNeg.C time, Num time, Ord body) =>
    time -> NonEmptyList time (time,body) -> Bool
 moveForwardRestrictedInfinity maxTime =
    checkInfinite .
@@ -769,16 +885,16 @@ moveForwardRestrictedInfinity maxTime =
 
 
 
-moveForwardLimited :: (NonNeg.C time) =>
+moveForwardLimited :: (NonNeg.C time, Num time) =>
    TimeTimeList.T time (time,body) -> TimeTimeList.T time body
 moveForwardLimited = TimeTimeList.moveForward . restrictMoveTimes
 
 restrictMoveTimes :: (NonNeg.C time) =>
    TimeTimeList.T time (time,body) -> TimeTimeList.T time (time,body)
 restrictMoveTimes =
-   flip evalState 0 .
+   flip evalState zero .
    TimeTimeList.mapM
-      (\t -> modify (t+) >> return t)
+      (\t -> modify (add t) >> return t)
       (\(t,b) -> gets (\tm -> (min t tm, b)))
 
 
@@ -788,7 +904,7 @@ arrangeSingletons evs =
    evs ==
    TimeTimeList.arrange
       (TimeTimeList.mapBody
-          (\x -> TimeTimeList.cons 0 x (TimeTimeList.pause 0)) evs)
+          (\x -> TimeTimeList.cons zero x (TimeTimeList.pause zero)) evs)
 
 arrangeDelay :: (NonNeg.C time, Ord body) =>
    time -> NonEmptyList time body -> Bool
@@ -797,8 +913,8 @@ arrangeDelay delay evs0 =
    in  TimeTimeList.delay delay evs ==
        TimeTimeList.arrange
           (TimeTimeList.mapBody
-              (\x -> TimeTimeList.cons delay x (TimeTimeList.pause 0)) $
-           TimeTimePriv.mapTimeLast (delay+) evs)
+              (\x -> TimeTimeList.cons delay x (TimeTimeList.pause zero)) $
+           TimeTimePriv.mapTimeLast (add delay) evs)
 
 arrangeSimple :: (NonNeg.C time, Ord body) =>
    TimeTimeList.T time (TimeTimeList.T time body) ->
@@ -809,10 +925,10 @@ arrangeSimple evs =
    TimeTimeList.foldr
       (TimeTimeList.delay)
       (TimeTimeList.merge)
-      (TimeTimeList.pause 0)
+      (TimeTimeList.pause zero)
       (TimeTimeList.mapBody TimeTimeList.normalize evs)
 
-arrangeAbsolute :: (NonNeg.C time, Ord body) =>
+arrangeAbsolute :: (NonNeg.C time, Num time, Ord body) =>
    TimeTimeList.T time (TimeTimeList.T time body) ->
    Bool
 arrangeAbsolute evs =
@@ -822,10 +938,10 @@ arrangeAbsolute evs =
           TimeTimeList.merge
              (TimeTimeList.delay t (TimeTimeList.normalize xs)) ys)
       (,)
-      (TimeTimeList.pause 0, TimeTimeList.pause 0)
-      (TimeTimeList.toAbsoluteEventList 0 evs)
+      (TimeTimeList.pause zero, TimeTimeList.pause zero)
+      (TimeTimeList.toAbsoluteEventList zero evs)
 
-arrangeInfinity :: (NonNeg.C time, Ord body) =>
+arrangeInfinity :: (NonNeg.C time, Num time, Ord body) =>
    NonEmptyList time (NonEmptyList time body) -> Bool
 arrangeInfinity =
    checkInfinite .
@@ -845,14 +961,14 @@ collectCoincidentGaps :: (NonNeg.C time, Eq body) =>
    TimeTimeList.T time body -> Bool
 collectCoincidentGaps xs =
    let times = tail (TimeTimeList.getTimes (TimeTimeList.collectCoincident xs))
-   in  null times || all (0<) (init times)
+   in  null times || all (zero<) (init times)
 
 collectCoincidentNonEmpty :: (NonNeg.C time, Eq body) =>
    TimeTimeList.T time body -> Bool
 collectCoincidentNonEmpty =
    all (not . null) . TimeTimeList.getBodies . TimeTimeList.collectCoincident
 
-collectCoincidentInfinite :: (NonNeg.C time, Eq body) =>
+collectCoincidentInfinite :: (NonNeg.C time, Num time, Eq body) =>
    NonEmptyList time body -> Bool
 collectCoincidentInfinite =
    checkInfinite .
@@ -975,7 +1091,7 @@ toEventList xs =
 discretize :: (RealFrac time, Integral i) =>
    TimeTimeList.T time body -> TimeTimeList.T i body
 discretize es =
-   evalState (Uniform.mapSecondM roundDiff es) 0
+   evalState (Uniform.mapSecondM roundDiff es) zero
 
 resample :: (RealFrac time, Integral i) =>
    time -> TimeTimeList.T time body -> TimeTimeList.T i body
@@ -988,7 +1104,7 @@ toAbsoluteEventList :: (Num time) =>
 toAbsoluteEventList start xs =
    let ts = Uniform.getSeconds xs
        bs = Uniform.getFirsts  xs
-       ats = List.scanl (+) start ts
+       ats = List.scanl add start ts
    in  maybe
           (error "padded list always contains one time value")
           (\ ~(ats0,lt) -> (zip ats0 bs, lt))
@@ -1000,11 +1116,12 @@ toAbsoluteEventList start xs =
 
 type NonEmptyList time body = (time, body, TimeTimeList.T time body)
 
-makeUncollapsedInfiniteEventList :: (NonNeg.C time) =>
+makeUncollapsedInfiniteEventList ::
+   (NonNeg.C time, Num time) =>
    NonEmptyList time body -> TimeTimeList.T time body
 makeUncollapsedInfiniteEventList =
    makeInfiniteEventList .
-   (\(time,body,xs) -> (time+1,body,xs))
+   (\(time,body,xs) -> (add time 1, body, xs))
 
 makeInfiniteEventList :: (NonNeg.C time) =>
    NonEmptyList time body -> TimeTimeList.T time body
@@ -1071,23 +1188,31 @@ tests =
      quickCheck (snocTimeBodyInfinite :: TimeDiff -> ArbChar -> NonEmptyList TimeDiff ArbChar -> Bool)) :
    ("consInfix",
      quickCheck (consInfix :: TimeDiff -> ArbChar -> TimeDiff -> TimeDiff -> ArbChar -> TimeDiff -> Bool)) :
+   ("chunkyShow",
+     quickCheck chunkyShow) :
+{-
+   ("chunkyCheck",
+     quickCheck chunkyCheck) :
+-}
+   ("mapTimeTailChunkyInfinite",
+     quickCheck (mapTimeTailChunkyInfinite succ :: BodyTimeList.T (NonNegChunky.T TimeDiff) ArbChar -> Bool)) :
 
 
    ("map body composition",
      quickCheck (mapBodyComposition toUpper toLower
                :: TimeTimeList.T TimeDiff ArbChar -> Bool)) :
    ("map time composition",
-     quickCheck ((\dt0 dt1 -> mapTimeComposition (dt0+) (dt1+))
+     quickCheck ((\dt0 dt1 -> mapTimeComposition (add dt0) (add dt1))
                :: TimeDiff -> TimeDiff -> TimeTimeList.T TimeDiff ArbChar -> Bool)) :
    ("map time body commutative",
-     quickCheck ((\dt -> mapTimeBodyCommutative (dt+) toUpper)
+     quickCheck ((\dt -> mapTimeBodyCommutative (add dt) toUpper)
                :: TimeDiff -> TimeTimeList.T TimeDiff ArbChar -> Bool)) :
 
    ("mapBodyInfinite",
      quickCheck (mapBodyInfinite toUpper
                :: NonEmptyList TimeDiff ArbChar -> Bool)) :
    ("mapTimeInfinite",
-     quickCheck (\dt -> mapTimeInfinite (dt+)
+     quickCheck (\dt -> mapTimeInfinite (add dt)
                :: NonEmptyList TimeDiff ArbChar -> Bool)) :
 
    ("mapNormalize",
@@ -1151,6 +1276,12 @@ tests =
      quickCheck (takeTimeInfinite :: TimeDiff -> NonEmptyList TimeDiff ArbChar -> Bool)) :
    ("dropTimeInfinite",
      quickCheck (dropTimeInfinite :: TimeDiff -> NonEmptyList TimeDiff ArbChar -> Bool)) :
+   ("dropTimeLargeInfinite",
+     quickCheck (dropTimeLargeInfinite :: NonEmptyList TimeDiff ArbChar -> Bool)) :
+   ("splitAtTimeLazyInfinite",
+     quickCheck (splitAtTimeLazyInfinite :: BodyTimeList.T (NonNegChunky.T TimeDiff) ArbChar -> Bool)) :
+   ("dropTimeLazyInfinite",
+     quickCheck (dropTimeLazyInfinite :: BodyTimeList.T (NonNegChunky.T TimeDiff) ArbChar -> Bool)) :
 
    ("duration pause",
      quickCheck (durationPause :: TimeDiff -> Bool)) :
